@@ -1,6 +1,7 @@
-from __future__ import absolute_import
-from . import main
+"""Default entry point for the schema-salad module."""
+
 import sys
-import typing
+
+from . import main
 
 sys.exit(main.main())
