@@ -1,9 +1,6 @@
-from __future__ import absolute_import
-
 import itertools
 import os
 import re
-import traceback
 from typing import (
     Any,
     AnyStr,
@@ -18,17 +15,11 @@ from typing import (
     Union,
 )
 
-import six
-from future.utils import raise_from
-from typing_extensions import Text  # pylint: disable=unused-import
-
 import ruamel.yaml
 from ruamel.yaml.comments import CommentedBase, CommentedMap, CommentedSeq
 
-# move to a regular typing import when Python 3.3-3.6 is no longer supported
 
-
-lineno_re = re.compile(u"^(.*?:[0-9]+:[0-9]+: )(( *)(.*))")
+lineno_re = re.compile("^(.*?:[0-9]+:[0-9]+: )(( *)(.*))")
 
 
 def regex_chunk(lines, regex):
@@ -66,19 +57,6 @@ def chunk_messages(message):  # type: (str) -> List[Tuple[int, str]]
     return arr
 
 
-def to_one_line_messages(message):  # type: (str) -> str
-    ret = []
-    max_elem = (0, "")
-    for (indent, msg) in chunk_messages(message):
-        if indent > max_elem[0]:
-            max_elem = (indent, msg)
-        else:
-            ret.append(max_elem[1])
-            max_elem = (indent, msg)
-    ret.append(max_elem[1])
-    return "\n".join(ret)
-
-
 def reformat_yaml_exception_message(message):  # type: (str) -> str
     line_regex = re.compile(r'^\s+in "(.+)", line (\d+), column (\d+)$')
     fname_regex = re.compile(r"^file://" + re.escape(os.getcwd()) + "/")
@@ -119,11 +97,11 @@ def _add_lc_filename(
         for d in r:
             _add_lc_filename(d, source)
     elif isinstance(r, MutableMapping):
-        for d in six.itervalues(r):
+        for d in r.values():
             _add_lc_filename(d, source)
 
 
-def relname(source):  # type: (Text) -> Text
+def relname(source):  # type: (str) -> str
     if source.startswith("file://"):
         source = source[7:]
         source = os.path.relpath(source)
@@ -132,11 +110,33 @@ def relname(source):  # type: (Text) -> Text
 
 def add_lc_filename(
     r, source
-):  # type: (ruamel.yaml.comments.CommentedBase, Text) -> None
+):  # type: (ruamel.yaml.comments.CommentedBase, str) -> None
     _add_lc_filename(r, relname(source))
 
 
-def reflow(text, maxline, shift=""):  # type: (Text, int, Text) -> Text
+def reflow_all(text, maxline=None):  # type: (str, Optional[int]) -> str
+    if maxline is None:
+        maxline = int(os.environ.get("COLUMNS", "100"))
+    maxno = 0
+    for l in text.splitlines():
+        g = lineno_re.match(l)
+        if not g:
+            continue
+        maxno = max(maxno, len(g.group(1)))
+    maxno_text = maxline - maxno
+    msg = []
+    for l in text.splitlines():
+        g = lineno_re.match(l)
+        if not g:
+            msg.append(l)
+            continue
+        pre = g.group(1)
+        reflowed = reflow(g.group(2), maxno_text, g.group(3)).splitlines()
+        msg.extend([pre.ljust(maxno, " ") + r for r in reflowed])
+    return "\n".join(msg)
+
+
+def reflow(text, maxline, shift=""):  # type: (str, int, str) -> str
     if maxline < 20:
         maxline = 20
     if len(text) > maxline:
@@ -153,30 +153,47 @@ def reflow(text, maxline, shift=""):  # type: (Text, int, Text) -> Text
 
 
 def indent(
-    v, nolead=False, shift=u"  ", bullet=u"  "
-):  # type: (Text, bool, Text, Text) -> Text
+    v, nolead=False, shift="  ", bullet="  "
+):  # type: (str, bool, str, str) -> str
     if nolead:
-        return v.splitlines()[0] + u"\n".join([shift + l for l in v.splitlines()[1:]])
+        return v.splitlines()[0] + "\n".join([shift + l for l in v.splitlines()[1:]])
     else:
 
-        def lineno(i, l):  # type: (int, Text) -> Text
+        def lineno(i, l):  # type: (int, str) -> str
             r = lineno_re.match(l)
             if r is not None:
                 return r.group(1) + (bullet if i == 0 else shift) + r.group(2)
             else:
                 return (bullet if i == 0 else shift) + l
 
-        return u"\n".join([lineno(i, l) for i, l in enumerate(v.splitlines())])
+        return "\n".join([lineno(i, l) for i, l in enumerate(v.splitlines())])
 
 
-def bullets(textlist, bul):  # type: (List[Text], Text) -> Text
+def bullets(textlist, bul):  # type: (List[str], str) -> str
     if len(textlist) == 1:
         return textlist[0]
     else:
         return "\n".join(indent(t, bullet=bul) for t in textlist)
 
 
-def strip_dup_lineno(text, maxline=None):  # type: (Text, Optional[int]) -> Text
+def strip_duplicated_lineno(text):  # type: (str) -> str
+    """Same as `strip_dup_lineno` but without reflow"""
+    pre = None
+    msg = []
+    for l in text.splitlines():
+        g = lineno_re.match(l)
+        if not g:
+            msg.append(l)
+            continue
+        elif g.group(1) != pre:
+            msg.append(l)
+            pre = g.group(1)
+        else:
+            msg.append(" " * len(g.group(1)) + g.group(2))
+    return "\n".join(msg)
+
+
+def strip_dup_lineno(text, maxline=None):  # type: (str, Optional[int]) -> str
     if maxline is None:
         maxline = int(os.environ.get("COLUMNS", "100"))
     pre = None
@@ -205,10 +222,10 @@ def strip_dup_lineno(text, maxline=None):  # type: (Text, Optional[int]) -> Text
 
 
 def cmap(
-    d,  # type: Union[int, float, str, Text, Dict[Text, Any], List[Dict[Text, Any]]]
+    d,  # type: Union[int, float, str, str, Dict[str, Any], List[Dict[str, Any]]]
     lc=None,  # type: Optional[List[int]]
-    fn=None,  # type: Optional[Text]
-):  # type: (...) -> Union[int, float, str, Text, CommentedMap, CommentedSeq]
+    fn=None,  # type: Optional[str]
+):  # type: (...) -> Union[int, float, str, str, CommentedMap, CommentedSeq]
     if lc is None:
         lc = [0, 0, 0, 0]
     if fn is None:
@@ -216,7 +233,7 @@ def cmap(
 
     if isinstance(d, CommentedMap):
         fn = d.lc.filename if hasattr(d.lc, "filename") else fn
-        for k, v in six.iteritems(d):
+        for k, v in d.items():
             if d.lc.data is not None and k in d.lc.data:
                 d[k] = cmap(v, lc=d.lc.data[k], fn=fn)
             else:
@@ -266,7 +283,7 @@ class SourceLine(object):
         self,
         item,  # type: Any
         key=None,  # type: Optional[Any]
-        raise_type=six.text_type,  # type: Union[Type[six.text_type], Type[Exception]]
+        raise_type=str,  # type: Union[Type[str], Type[Exception]]
         include_traceback=False,  # type: bool
     ):  # type: (...) -> None
         self.item = item
@@ -285,37 +302,41 @@ class SourceLine(object):
     ):  # type: (...) -> None
         if not exc_value:
             return
-        if self.include_traceback and six.PY2:
-            # Python2 doesn't actually have chained exceptions, so
-            # fake it by injecting the backtrace into the message.
-            raise_from(
-                self.makeError(
-                    "\n".join(traceback.format_exception(exc_type, exc_value, tb))
-                ),
-                exc_value,
-            )
-        else:
-            raise_from(self.makeError(six.text_type(exc_value)), exc_value)
+        raise self.makeError(str(exc_value)) from exc_value
 
-    def makeLead(self):  # type: () -> Text
-        if (
+    def file(self):  # type: () -> Optional[str]
+        if hasattr(self.item, "lc") and hasattr(self.item.lc, "filename"):
+            return str(self.item.lc.filename)
+        else:
+            return None
+
+    def start(self):  # type: () -> Optional[Tuple[int, int]]
+        if self.file() is None:
+            return None
+        elif (
             self.key is None
             or self.item.lc.data is None
             or self.key not in self.item.lc.data
         ):
-            return "{}:{}:{}:".format(
-                self.item.lc.filename if hasattr(self.item.lc, "filename") else "",
-                (self.item.lc.line or 0) + 1,
-                (self.item.lc.col or 0) + 1,
-            )
+            return ((self.item.lc.line or 0) + 1, (self.item.lc.col or 0) + 1)
         else:
-            return "{}:{}:{}:".format(
-                self.item.lc.filename if hasattr(self.item.lc, "filename") else "",
+            return (
                 (self.item.lc.data[self.key][0] or 0) + 1,
                 (self.item.lc.data[self.key][1] or 0) + 1,
             )
 
-    def makeError(self, msg):  # type: (Text) -> Any
+    def end(self):  # type: () -> Optional[Tuple[int, int]]
+        return None
+
+    def makeLead(self):  # type: () -> str
+        if self.file():
+            lcol = self.start()
+            line, col = lcol if lcol else ("", "")
+            return "{}:{}:{}:".format(self.file(), line, col)
+        else:
+            return ""
+
+    def makeError(self, msg):  # type: (str) -> Any
         if not isinstance(self.item, ruamel.yaml.comments.CommentedBase):
             return self.raise_type(msg)
         errs = []
