@@ -1,7 +1,4 @@
-from __future__ import absolute_import, print_function
-
 import pytest
-import six
 
 import schema_salad
 from schema_salad.schema import load_and_validate, load_schema
@@ -12,7 +9,7 @@ from .util import get_data
 
 def test_errors():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     for t in (
@@ -32,19 +29,16 @@ def test_errors():
         with pytest.raises(ValidationException):
             try:
                 load_and_validate(
-                    document_loader,
-                    avsc_names,
-                    six.text_type(get_data("tests/" + t)),
-                    True,
+                    document_loader, avsc_names, str(get_data("tests/" + t)), True,
                 )
             except ValidationException as e:
-                print ("\n", e)
+                print("\n", e)
                 raise
 
 
 def test_error_message1():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test1.cwl"
@@ -57,13 +51,13 @@ def test_error_message1():
     )
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message2():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test2.cwl"
@@ -74,13 +68,13 @@ def test_error_message2():
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message3():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test3.cwl"
@@ -93,13 +87,13 @@ def test_error_message3():
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message4():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test4.cwl"
@@ -111,76 +105,76 @@ def test_error_message4():
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message5():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test5.cwl"
     match = r"""
 ^.+test5\.cwl:2:1: Object\s+`.+test5\.cwl`\s+is\s+not valid because
 \s+tried `Workflow`\s+but
-.+test5\.cwl:7:1:     the `steps`\s+field\s+is\s+not valid\s+because
+.+test5\.cwl:7:1:     the `steps`\s+field\s+is\s+not\s+valid\s+because
 \s+tried array\s+of\s+<WorkflowStep>\s+but
-.+test5\.cwl:7:9:         item is\s+invalid because
+.+test5\.cwl:7:9:         item is\s+invalid\s+because
 \s+is not a\s+dict$"""[
         1:
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message7():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test7.cwl"
     match = (
         r"""^.+test7\.cwl:2:1: Object\s+`.+test7\.cwl`\s+is\s+not valid because
 \s+tried `Workflow`\s+but
-.+test7\.cwl:7:1:     the `steps`\s+field\s+is\s+not valid\s+because
+.+test7\.cwl:7:1:     the `steps`\s+field\s+is\s+not\s+valid\s+because
 \s+tried array\s+of\s+<WorkflowStep>\s+but
-.+test7\.cwl:8:3:         item is\s+invalid because
-\s+\* missing\s+required\s+field `run`
-.+test7\.cwl:9:5:           \* invalid\s+field\s+`scatter_method`,\s+expected one """
-        + r"""of:\s+'id', 'in', 'out',\s+'requirements',\s+'hints', """
+.+test7\.cwl:8:3:         item is\s+invalid\s+because
+\s+\* missing\s+required\s+field\s+`run`
+.+test7\.cwl:9:5:           \* invalid\s+field\s+`scatter_method`,\s+expected\s+one """
+        + r"""of:\s+'id',\s+'in', 'out',\s+'requirements',\s+'hints',\s+"""
         + r"""'label',\s+'doc',\s+'run',\s+'scatter',\s+'scatterMethod'$"""
     )
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message8():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test8.cwl"
     match = r"""
 ^.+test8\.cwl:7:1: checking field\s+`steps`
 .+test8\.cwl:8:3:   checking object\s+`.+test8\.cwl#step1`
-.+test8\.cwl:9:5:     Field\s+`scatterMethod`\s+contains\s+undefined\s+reference to
+.+test8\.cwl:9:5:     Field\s+`scatterMethod`\s+contains\s+undefined\s+reference\s+to
 \s+`file:///.+/tests/test_schema/abc`$"""[
         1:
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message9():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test9.cwl"
@@ -188,40 +182,40 @@ def test_error_message9():
         r"""^.+test9\.cwl:7:1: checking field\s+`steps`
 .+test9\.cwl:8:3:   checking object\s+`.+test9\.cwl#step1`
 .+test9\.cwl:9:5:     `scatterMethod`\s+field\s+is\s+"""
-        + r"""int,\s+expected\s+string,\s+list, or a dict.$"""
+        + r"""int,\s+expected\s+string,\s+list,\s+or a\s+dict.$"""
     )
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message10():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test10.cwl"
     match = r"""
 ^.+test10\.cwl:2:1: Object\s+`.+test10\.cwl`\s+is not valid because
 \s+tried `Workflow`\s+but
-.+test10\.cwl:7:1:     the `steps`\s+field is\s+not valid\s+because
+.+test10\.cwl:7:1:     the `steps`\s+field\s+is\s+not\s+valid\s+because
 \s+tried array\s+of\s+<WorkflowStep>\s+but
-.+test10\.cwl:8:3:         item is\s+invalid because
-\s+\* missing\s+required\s+field `run`
-.+test10\.cwl:9:5:           \* the\s+`scatterMethod`\s+field is\s+not valid\s+because
-\s+value\s+is a\s+CommentedSeq,\s+expected\s+null\s+or\s+ScatterMethod$"""[
+.+test10\.cwl:8:3:         item is\s+invalid\s+because
+\s+\* missing\s+required\s+field\s+`run`
+.+test10\.cwl:9:5:           \* the\s+`scatterMethod`\s+field\s+is\s+not\s+valid\s+because
+\s+value\s+is\s+a\s+CommentedSeq,\s+expected\s+null\s+or\s+ScatterMethod$"""[
         1:
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message11():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test11.cwl"
@@ -234,33 +228,33 @@ def test_error_message11():
     ]
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
 def test_error_message15():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     t = "test_schema/test15.cwl"
     match = (
         r"""^.+test15\.cwl:3:1:\s+Object\s+`.+test15\.cwl`\s+is not valid because
 \s+tried\s+`CommandLineTool`\s+but
-.+test15\.cwl:6:1:\s+the `inputs`\s+field\s+is not valid\s+because
+.+test15\.cwl:6:1:\s+the `inputs`\s+field\s+is\s+not valid\s+because
 .+test15\.cwl:7:3:\s+item is\s+invalid\s+because
-.+test15\.cwl:9:5:\s+the\s+`inputBinding`\s+field is not\s+valid\s+because
+.+test15\.cwl:9:5:\s+the\s+`inputBinding`\s+field\s+is\s+not\s+valid\s+because
 .+tried\s+CommandLineBinding\s+but
-.+test15\.cwl:11:7:             \*\s+invalid field\s+`invalid_field`,\s+expected """
+.+test15\.cwl:11:7:             \*\s+invalid\s+field\s+`invalid_field`,\s+expected\s+"""
         + r"""one\s+of:\s+'loadContents',\s+'position',\s+'prefix',\s+'separate',"""
         + r"""\s+'itemSeparator',\s+'valueFrom',\s+'shellQuote'
-.+test15\.cwl:12:7:             \*\s+invalid field\s+`another_invalid_field`,"""
+.+test15\.cwl:12:7:             \*\s+invalid\s+field\s+`another_invalid_field`,"""
         + r"""\s+expected one\s+of:\s+'loadContents',\s+'position',\s+'prefix',"""
         + r"""\s+'separate',\s+'itemSeparator',\s+'valueFrom',\s+'shellQuote'$"""
     )
     with pytest.raises(ValidationException, match=match):
         load_and_validate(
-            document_loader, avsc_names, six.text_type(get_data("tests/" + t)), True
+            document_loader, avsc_names, str(get_data("tests/" + t)), True
         )
 
 
@@ -269,7 +263,7 @@ def test_error_message15():
 )
 def test_errors_previously_defined_dict_key():
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     for t in (
@@ -280,13 +274,10 @@ def test_errors_previously_defined_dict_key():
         with pytest.raises(ValidationException):
             try:
                 load_and_validate(
-                    document_loader,
-                    avsc_names,
-                    six.text_type(get_data("tests/" + t)),
-                    True,
+                    document_loader, avsc_names, str(get_data("tests/" + t)), True,
                 )
             except ValidationException as e:
-                print ("\n", e)
+                print("\n", e)
                 raise
 
 
