@@ -1,14 +1,10 @@
-from __future__ import absolute_import, print_function
-
 import os
 
-import six
-
 import ruamel.yaml
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
 import schema_salad.main
 import schema_salad.ref_resolver
 import schema_salad.schema
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.jsonld_context import makerdf
 from schema_salad.sourceline import SourceLine, cmap
 
@@ -21,20 +17,20 @@ def test_schemas():
     ra, _ = loader.resolve_all(
         cmap(
             {
-                u"$schemas": [
+                "$schemas": [
                     schema_salad.ref_resolver.file_uri(get_data("tests/EDAM.owl"))
                 ],
-                u"$namespaces": {u"edam": u"http://edamontology.org/"},
-                u"edam:has_format": u"edam:format_1915",
+                "$namespaces": {"edam": "http://edamontology.org/"},
+                "edam:has_format": "edam:format_1915",
             }
         ),
         "",
     )
 
     assert {
-        u"$schemas": [schema_salad.ref_resolver.file_uri(get_data("tests/EDAM.owl"))],
-        u"$namespaces": {u"edam": u"http://edamontology.org/"},
-        u"http://edamontology.org/has_format": u"http://edamontology.org/format_1915",
+        "$schemas": [schema_salad.ref_resolver.file_uri(get_data("tests/EDAM.owl"))],
+        "$namespaces": {"edam": "http://edamontology.org/"},
+        "http://edamontology.org/has_format": "http://edamontology.org/format_1915",
     } == ra
 
 
@@ -272,7 +268,7 @@ def test_scoped_id():
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print (g.serialize(format="n3"))
+    print(g.serialize(format="n3"))
 
     ra, _ = ldr.resolve_all(
         cmap({"location": "foo", "bar": {"location": "baz"}}),
@@ -285,7 +281,7 @@ def test_scoped_id():
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print (g.serialize(format="n3"))
+    print(g.serialize(format="n3"))
 
     ra, _ = ldr.resolve_all(
         cmap({"id": "foo", "bar": {"location": "baz"}}),
@@ -298,7 +294,7 @@ def test_scoped_id():
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print (g.serialize(format="n3"))
+    print(g.serialize(format="n3"))
 
     ra, _ = ldr.resolve_all(
         cmap({"location": "foo", "bar": {"id": "baz"}}),
@@ -311,7 +307,7 @@ def test_scoped_id():
     } == ra
 
     g = makerdf(None, ra, ctx)
-    print (g.serialize(format="n3"))
+    print(g.serialize(format="n3"))
 
 
 def test_subscoped_id():
@@ -347,8 +343,8 @@ def test_mixin():
         base_url=base_url,
     )
     assert [
-        {"id": base_url + "#a", "m": {"id": base_url + u"#a/four", "one": "two"}},
-        {"id": base_url + "#b", "m": {"id": base_url + u"#b/four", "one": "two"}},
+        {"id": base_url + "#a", "m": {"id": base_url + "#a/four", "one": "two"}},
+        {"id": base_url + "#b", "m": {"id": base_url + "#b/four", "one": "two"}},
     ] == ra[0]
 
 
@@ -399,19 +395,6 @@ def test_sourceline():
     except Exception as exc:
         assert False, exc
 
-    if six.PY2:
-        try:
-            with SourceLine(b, 1, TestExp, True):
-                raise Exception("Whoops")
-        except TestExp as e:
-            assert (
-                str(e)
-                .splitlines()[0]
-                .endswith("frag.yml:3:3: Traceback (most recent call last):")
-            ), str(e)
-        except Exception as exc:
-            assert False, exc
-
 
 def test_cmap():
     # Test bugfix that cmap won't fail when given a CommentedMap with no lc.data
@@ -432,3 +415,15 @@ def test_blank_node_id():
 
     ra, _ = ldr.resolve_all(cmap({"id": "_:foo"}), "http://example.com")
     assert {"id": "_:foo"} == ra
+
+
+def test_can_use_Any():
+    # Test that 'type: Any' can be used
+    (
+        document_loader,
+        avsc_names,
+        schema_metadata,
+        metaschema_loader,
+    ) = schema_salad.schema.load_schema(  # noqa: B950
+        get_data("tests/test_schema/cwltest-schema.yml")
+    )
