@@ -1,12 +1,10 @@
+import re
 from os.path import normpath
 
 import pytest
-import six
 
-from schema_salad.main import reformat_yaml_exception_message, to_one_line_messages
+from schema_salad.exceptions import ValidationException, to_one_line_messages
 from schema_salad.schema import load_and_validate, load_schema
-from schema_salad.sourceline import strip_dup_lineno
-from schema_salad.validate import ValidationException
 
 from .util import get_data
 
@@ -14,7 +12,7 @@ from .util import get_data
 def test_print_oneline():
     # Issue #135
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test15.cwl"
@@ -23,11 +21,11 @@ def test_print_oneline():
             load_and_validate(
                 document_loader,
                 avsc_names,
-                six.text_type(get_data("tests/test_schema/" + src)),
+                str(get_data("tests/test_schema/" + src)),
                 True,
             )
         except ValidationException as e:
-            msgs = to_one_line_messages(str(e)).splitlines()
+            msgs = to_one_line_messages(e).splitlines()
             assert len(msgs) == 2
             assert msgs[0].endswith(
                 src + ":11:7: invalid field `invalid_field`, expected one of: "
@@ -39,37 +37,36 @@ def test_print_oneline():
                 "'loadContents', 'position', 'prefix', 'separate', 'itemSeparator', "
                 "'valueFrom', 'shellQuote'"
             )
-            print ("\n", e)
+            print("\n", e)
             raise
 
 
 def test_print_oneline_for_invalid_yaml():
     # Issue #137
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test16.cwl"
-    with pytest.raises(RuntimeError):
+    with pytest.raises(ValidationException):
         try:
             load_and_validate(
                 document_loader,
                 avsc_names,
-                six.text_type(get_data("tests/test_schema/" + src)),
+                str(get_data("tests/test_schema/" + src)),
                 True,
             )
-        except RuntimeError as e:
-            msg = reformat_yaml_exception_message(strip_dup_lineno(six.text_type(e)))
-            msg = to_one_line_messages(msg)
+        except ValidationException as e:
+            msg = to_one_line_messages(e)
             assert msg.endswith(src + ":11:1: could not find expected ':'")
-            print ("\n", e)
+            print("\n", e)
             raise
 
 
 def test_print_oneline_for_errors_in_the_same_line():
     # Issue #136
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test17.cwl"
@@ -78,11 +75,11 @@ def test_print_oneline_for_errors_in_the_same_line():
             load_and_validate(
                 document_loader,
                 avsc_names,
-                six.text_type(get_data("tests/test_schema/" + src)),
+                str(get_data("tests/test_schema/" + src)),
                 True,
             )
         except ValidationException as e:
-            msgs = to_one_line_messages(str(e)).splitlines()
+            msgs = to_one_line_messages(e).splitlines()
             assert len(msgs) == 2, msgs
             assert msgs[0].endswith(src + ":14:5: missing required field `id`")
             assert msgs[1].endswith(
@@ -90,34 +87,28 @@ def test_print_oneline_for_errors_in_the_same_line():
                 "'secondaryFiles', 'format', 'streamable', 'doc', 'id', "
                 "'outputBinding', 'type'"
             )
-            print ("\n", e)
+            print("\n", e)
             raise
 
 
 def test_print_oneline_for_errors_in_resolve_ref():
     # Issue #141
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test18.cwl"
     fullpath = normpath(get_data("tests/test_schema/" + src))
     with pytest.raises(ValidationException):
         try:
-            load_and_validate(
-                document_loader, avsc_names, six.text_type(fullpath), True
-            )
+            load_and_validate(document_loader, avsc_names, str(fullpath), True)
         except ValidationException as e:
-            msgs = to_one_line_messages(
-                str(strip_dup_lineno(six.text_type(e)))
-            ).splitlines()
+            msg = to_one_line_messages(e)
             # convert Windows path to Posix path
             if "\\" in fullpath:
                 fullpath = "/" + fullpath.lower().replace("\\", "/")
-            assert len(msgs) == 2
-            print ("\n", e)
-            assert msgs[0].endswith(src + ":9:1: checking field `outputs`")
-            assert msgs[1].endswith(
+            print("\n", e)
+            assert msg.endswith(
                 src + ":14:5: Field `type` references unknown identifier "
                 "`Filea`, tried file://%s#Filea" % (fullpath)
             )
@@ -127,58 +118,49 @@ def test_print_oneline_for_errors_in_resolve_ref():
 def test_for_invalid_yaml1():
     # Issue 143
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test16.cwl"
-    with pytest.raises(RuntimeError):
+    with pytest.raises(ValidationException):
         try:
             load_and_validate(
                 document_loader,
                 avsc_names,
-                six.text_type(get_data("tests/test_schema/" + src)),
+                str(get_data("tests/test_schema/" + src)),
                 True,
             )
-        except RuntimeError as e:
-            msg = reformat_yaml_exception_message(strip_dup_lineno(six.text_type(e)))
-            msgs = msg.splitlines()
-            assert len(msgs) == 2
-            assert msgs[0].endswith(src + ":10:7: while scanning a simple key")
-            assert msgs[1].endswith(src + ":11:1:   could not find expected ':'")
-            print ("\n", e)
+        except ValidationException as e:
+            msg = str(e)
+            assert re.search(src + r":10:7: while scanning a\s+simple\s+key", msg, re.M)
+            assert re.search(
+                src + r":11:1:   could not\s+find\s+expected ':'$", msg, re.M
+            )
+            print("\n", e)
             raise
 
 
 def test_for_invalid_yaml2():
     # Issue 143
     document_loader, avsc_names, schema_metadata, metaschema_loader = load_schema(
-        get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     )
 
     src = "test19.cwl"
-    try:
-        load_and_validate(
-            document_loader,
-            avsc_names,
-            six.text_type(get_data("tests/test_schema/" + src)),
-            True,
-        )
-    except RuntimeError as e:
-        msg = reformat_yaml_exception_message(strip_dup_lineno(six.text_type(e)))
-        assert msg.endswith(
-            src + ":2:1: expected <block end>, but found ':'"
-        ) or msg.endswith(src + ":2:1: expected <block end>, but found u':'")
-        return
-    except ValidationException as e:
-        msgs = str(strip_dup_lineno(six.text_type(e)))
-        print (msgs)
-        # weird splits due to differing path length on MS Windows
-        # & during the release tests
-        assert "{}:2:1: Object".format(src) in msgs
-        assert "is not valid because" in msgs
-        assert "`CommandLineTool`" in msgs
-        assert "mapping with" in msgs
-        assert "implicit" in msgs
-        assert "null key" in msgs
-        return
-    assert False, "Missing RuntimeError or ValidationException"
+    with pytest.raises(ValidationException):
+        try:
+            load_and_validate(
+                document_loader,
+                avsc_names,
+                str(get_data("tests/test_schema/" + src)),
+                True,
+            )
+        except ValidationException as e:
+            msg = str(e)
+            assert (
+                msg.endswith("expected <block end>, but found ':'")
+                or msg.endswith("expected <block end>, but found u':'")
+                or re.search(r"mapping with\s+implicit\s+null key$", msg, re.M)
+            )
+            print("\n", e)
+            raise
