@@ -5,7 +5,6 @@ run individually as py.test -k tests/test_real_cwl.py
 """
 
 import pytest
-import six
 
 from schema_salad.schema import load_and_validate, load_schema
 from schema_salad.validate import ValidationException
@@ -18,8 +17,13 @@ test_dir_name = "tests/test_real_cwl/"
 class TestRealWorldCWL:
     @classmethod
     def setup_class(cls):
-        cls.document_loader, cls.avsc_names, schema_metadata, metaschema_loader = load_schema(  # noqa: B950
-            get_data(u"tests/test_schema/CommonWorkflowLanguage.yml")
+        (
+            cls.document_loader,
+            cls.avsc_names,
+            schema_metadata,
+            metaschema_loader,
+        ) = load_schema(  # noqa: B950
+            get_data("tests/test_schema/CommonWorkflowLanguage.yml")
         )
 
     def load_cwl(self, src):
@@ -28,12 +32,12 @@ class TestRealWorldCWL:
                 load_and_validate(
                     self.document_loader,
                     self.avsc_names,
-                    six.text_type(get_data(test_dir_name + src)),
+                    str(get_data(test_dir_name + src)),
                     True,
                 )
             except ValidationException as e:
                 # msgs = to_one_line_messages(str(e)).splitlines()
-                print ("\n", e)
+                print("\n", e)
                 raise
 
     def test_topmed_single_doc(self):
